function [rpoles, rres, cpoles, cres, gain] = pz2pr(poles, zeros, FreqGain, MagGain, period);
% Given the poles and the zeros of a discrete time system poles, zeros,
% this function calculates the residues and the gain.  
% Inputs:
%    poles : Vector of poles (complex) (both poles of a complex pair are present)
%    zeros :  ''       zeros    ''       ''     ''         ''         ''   ''
%    FreqGain: Frequency at which the magnitude of the transfer function is specified 
%    MagGain: Magnitude squared at at frequecny FreqGain
%    period: clock period (used only in evaluating gain 
% Outputs
%    rpoles : vector of real poles
%    rres : vector of real residues
%    cpoles: vector of complex poles (only one from every complex pair is present)
%    cres: vector of complex residues ( '' ''             ''                 ''  )
%    gain: filter gain

np = length(poles);
nz = length(zeros);
mm = min(np, nz);

for i=1:np,
  r = 1;
  x = poles(i);
  for k=1:mm,
    if (k~=i) r = r*(x-zeros(k))/(x-poles(k)); 
      else r = r * (x-zeros(k));
    end;
  end;
  for k=mm+1:np,
    if (k~=i) r = r/(x-poles(k) ); end;
  end;
  for k=mm+1:nz,
    r = r*(x-zeros(k) ); 
  end;
  for k = nz+1 : np,
    r = r * (x+1);
  end;
  res(i) = r;
end;

nr = 0; nc=0;
cpoles=[]; rpoles=[];
for i=1:np,
  x = poles(i);
  if (  abs(angle(x)) <1e-6 | abs(angle(-x))<1e-6 ) nr=nr+1; rpoles(nr) = real(x);
    else
      found = 0;
      for k=1:length(cpoles),
        if (x == conj(cpoles(k))) found=1; end;
      end;
      if (found == 0) nc=nc+1; cpoles(nc) = x; end;
  end;
end;

nr = 0; nc=0;
cres=[]; rres=[];
for i=1:np,
  x = res(i);
  if ( abs(angle(x)) <1e-6 | abs(angle(-x))<1e-6 )  nr=nr+1; rres(nr) = real(x); 
    else
      found = 0;
      for k=1:length(cres),
        if (x == conj(cres(k))) found=1; end;
      end;
      if (found == 0) nc=nc+1; cres(nc) = x; end;
  end;
end;

% For Bilinear:
srp=1; 
x = exp(j*2*pi*FreqGain*period);
for i=1:nr,
  srp = srp + rres(i)./(x -rpoles(i));
end;
for i=1:nc,
  srp = srp + 2*real(cres(i)./(x -cpoles(i)));
end;
gain = sqrt(MagGain)/abs(srp);

